"""
Views
=====

This module contains the views of the PEtab edit GUI, namely
- the main window
- the SBML viewer
- the PEtab table viewers
- the task bar
- the log window
"""

from .main_view import MainWindow
from .sbml_view import SbmlViewer
from .table_view import TableViewer
from .task_bar import TaskBar
